/* 
 * File:   test.c
 * Author: hardi_000
 *
 * Created on June 25, 2013, 1:48 PM
 */
/*PIC Pinout configuration*/
/*
 *  PIN 1   MCLR
 *  PIN 2   UART1 TX
 *  PIN 3   RGB - Blue  (RA1)
 *  PIN 4   PGED1 - SD Card chip select
 *  PIN 5   PGEC1 - Shutdown pin
 *  PIN 6   SDI2        (RB2)
 *  PIN 7   RGB - Red   (RB3)
 *  PIN 8   Vss
 *  PIN 9   OSC1
 *  PIN 10  OSC2
 *  PIN 11  ########    (RB4) Secondary CLK in?
 *  PIN 12  UART1 RX
 *  PIN 13  Vdd
 *  PIN 14  RFM22B - TX (RB5)
 *  PIN 15  VBUS
 *  PIN 16  RFM22B - CS
 *  PIN 17  SDO2        (RB8)
 *  PIN 18  RFM22B - RX
 *  PIN 19  Vss
 *  PIN 20  Vcap
 *  PIN 21  USB - D+
 *  PIN 22  USB - D-
 *  PIN 23  VUSB 3V3
 *  PIN 24  RGB - Green (RB13)
 *  PIN 25  VUSBON
 *  PIN 26  SCK2
 *  PIN 27  AVss
 *  PIN 28  AVdd
 */


#include <stdio.h>
#include <stdlib.h>

#include <xc.h>
#include <plib.h>
#include <math.h>
//#include <string.h>

// PIC32MX220F032B Configuration Bit Settings

#include <p32xxxx.h>

// DEVCFG3
// USERID = No Setting
#pragma config PMDL1WAY = ON            // Peripheral Module Disable Configuration (Allow only one reconfiguration)
#pragma config IOL1WAY = ON             // Peripheral Pin Select Configuration (Allow only one reconfiguration)
#pragma config FUSBIDIO = ON            // USB USID Selection (Controlled by the USB Module)
#pragma config FVBUSONIO = ON           // USB VBUS ON Selection (Controlled by USB Module)

// DEVCFG2
#pragma config FPLLIDIV = DIV_5         // PLL Input Divider (5x Divider)
#pragma config FPLLMUL = MUL_20         // PLL Multiplier (20x Multiplier)
#pragma config UPLLIDIV = DIV_1         // USB PLL Input Divider (1x Divider)
#pragma config UPLLEN = OFF             // USB PLL Enable (Disabled and Bypassed)
#pragma config FPLLODIV = DIV_2         // System PLL Output Clock Divider (PLL Divide by 2)

// DEVCFG1
#pragma config FNOSC = PRIPLL           // Oscillator Selection Bits (Primary Osc w/PLL (XT+,HS+,EC+PLL))
#pragma config FSOSCEN = OFF            // Secondary Oscillator Enable (Disabled)
#pragma config IESO = OFF               // Internal/External Switch Over (Disabled)
#pragma config POSCMOD = HS             // Primary Oscillator Configuration (HS osc mode)
#pragma config OSCIOFNC = ON            // CLKO Output Signal Active on the OSCO Pin (Enabled)
#pragma config FPBDIV = DIV_1           // Peripheral Clock Divisor (Pb_Clk is Sys_Clk/8)
#pragma config FCKSM = CSDCMD           // Clock Switching and Monitor Selection (Clock Switch Disable, FSCM Disabled)
#pragma config WDTPS = PS1048576        // Watchdog Timer Postscaler (1:1048576)
#pragma config WINDIS = OFF             // Watchdog Timer Window Enable (Watchdog Timer is in Non-Window Mode)
#pragma config FWDTEN = OFF             // Watchdog Timer Enable (WDT Disabled (SWDTEN Bit Controls))
#pragma config FWDTWINSZ = WISZ_25      // Watchdog Timer Window Size (Window Size is 25%)

// DEVCFG0
#pragma config JTAGEN = OFF             // JTAG Enable (JTAG Disabled)
#pragma config ICESEL = ICS_PGx1        // ICE/ICD Comm Channel Select (Communicate on PGEC1/PGED1)
#pragma config PWP = OFF                // Program Flash Write Protect (Disable)
#pragma config BWP = OFF                // Boot Flash Write Protect bit (Protection Disabled)
#pragma config CP = OFF                 // Code Protect (Protection Disabled)

//System clock setup at         50MHz
//Peripheral clock setup at     50MHz
#define SCLOCK 50000000

//Protocol definitions
#define COMMAND_LENGTH 28

//LED States
#define STATE_IDLE      0
#define STATE_SWEEP     1
#define STATE_DISCRETE  2
#define STATE_ERROR     3

#define DISCRETE_FLASH_PERIOD 6

//Buffer State
#define BUFFER_IDLE     0
#define BUFFER_READY    1
#define BUFFER_ERROR    2


void delay_us(int i);
void delay_ms(int i);

void initCommunications();
void initStatusLED();
void initRadio();
void setRGBState(int state);
void checkBufferState();
void checkSPIBufferState();

char readRFM22BRegister(int address);
void writeRFM22BRegister(char data,char address);
int setIFFilter(double rbw);
void setRadioFrequency(double freq);
void getRSSI();
void UARTSendDataString(const int id,const char *string);
void transmitToHost(char* string);
void receiveFromHost();

/*Global variables*/
long pbBusFreq = 50000000;

char receiveBuffer[COMMAND_LENGTH];
char spiBuffer = '0';
int count = 0;
int spiCount = 0;
int bufferState = BUFFER_IDLE;
int spiBufferState = BUFFER_IDLE;
int ledState = STATE_IDLE;

int discreteFlashCounter = 0;
int distreteFlashState = 0;
int ledIntensity = 0;
int ledStep = 1500;
char up = 1;

double startFrequency  = 0;
double stopFrequency = 0;
double currentFrequency = 0;
double freqStep = 0;
int noOfSteps = 1000;
int dwellTime = 10;
int currentSample = 0;
double maxSample = -130;
double minSample = 0;
double averageSample = 0;
char frequencyHasBeenSet = 0;
double rbw = 0;

char lnaGain = 0;
char progGain = 0;

/*Lookup table for if register settings*/
double IFfilterLookup[] =      {2.6    ,2.8    ,3.1    ,3.2    ,3.7    ,4.2    ,4.5    ,4.9,5.4,5.9,6.1,7.2,8.2,8.8,9.5,10.6,11.5,12.1,14.2,16.2,17.5,18.9,21,22.7,24.0,28.2,32.2,34.7,37.7,41.7,45.2,47.9,56.2,64.1,69.2,75.2,83.2,90,95.3,112.1,127.9,137.9,142.8,167.8,181.1,191.5,225.1,248.8,269.3,284.9,335.5,361.8,420.2,468.4,518.8,577.0,620.7};
char IFfilterNDecExp[] =    {5      ,5      ,5      ,5      ,5      ,5      ,5      ,4,4,4,4,4,4,4,3,3,3,3,3,3,3,2,2,2,2,2,2,2,1,1,1,1,1,1,1,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0};
char IFfilterDwn3Bypass[] = {0      ,0      ,0      ,0      ,0      ,0      ,0      ,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1};
char IFfilterFilset[] =     {1      ,2      ,3      ,4      ,5      ,6      ,7      ,1,2,3,4,5,6,7,1,2,3,4,5,6,7,1,2,3,4,5,6,7,1,2,3,4,5,6,7,1,2,3,4,5,6,7,4,5,9,15,1,2,3,4,8,9,10,11,12,13,14};

//<D.550000000.010600.000000100>
//<S.1.0.11.###################>
/* TODO:
 * scale wait time to rbw
 * add calibration
 * speed up sweep
 * add SO into pic for rtcc
 * setup and test rtcc
 * add sd card circuit
 * write and read functions for sd
 * add usb and test
 * relay functions from usb to normal functions
 * add battery
 * add battery voltage measure detector
 * add power swapover circuit
 * add regulator circuit for 12V external in
 */
int main(int argc, char** argv)
{
    /*Setup system internals*/
    pbBusFreq = SYSTEMConfig(50000000, SYS_CFG_ALL);
    
    /*Wait for all subsystems to power up*/
    delay_ms(250);
   

    /*Enable interrupt vectors*/
    INTConfigureSystem(INT_SYSTEM_CONFIG_MULT_VECTOR);
    INTEnableInterrupts();

    /*Initialize systems*/
    initCommunications();

    
//    debugRadioRead();
    transmitToHost("[System started]");
    while (1)
    {
        checkBufferState();        
        getRSSI();
    }
    return (EXIT_SUCCESS);
}

// UART Write String Function
void UARTSendDataString(const int id,const char *string)
{
     while(*string != '\0')
     {
        while(!UARTTransmitterIsReady(id));
        UARTSendDataByte(id, *string);
        string++;
        while(!UARTTransmissionHasCompleted(id));
     }
}


/*Acording to the state of the device queries the radio for
 its signal level and sets the frequency accordingly*/
void getRSSI()
{
    unsigned char rawRSSI;
    double unCalRssi;
    double calRssi;
    char packet[30];
    switch(ledState)
    {
        case STATE_SWEEP:
            /*Sets the radio to next frequency*/
            setRadioFrequency(currentFrequency);
            delay_ms(4); //Wait for PLL to settle
            /*Reads RSSI*/  
            rawRSSI = readRFM22BRegister(0x26);
            /*Changes rssi bit value to dBm*/
            unCalRssi = 0.5*rawRSSI - 122.5;
            /*Apply calibration*/
            calRssi = unCalRssi;
            /*Send RSSI to host with bin identifier*/
            sprintf(&packet,"%.6f %.1f",currentFrequency,calRssi);
            transmitToHost(packet);
            /*Step Frequency*/
            currentFrequency += freqStep;
            if(currentFrequency > stopFrequency)
            {
                ledState = STATE_IDLE;
                setRGBState(STATE_IDLE);
                receiveBuffer[0] = "\0";
                count = 0;
            }
            break;
        case STATE_DISCRETE:

            /*Check if the radio is on specified channel, 
             * if not set the frequency*/
            if(!frequencyHasBeenSet)
            {
                setRadioFrequency(currentFrequency);
                frequencyHasBeenSet = 1;
                delay_ms(2); //Wait for PLL to settle

            }
            /*Turn on dwell timer*/
            if(currentSample == 0)
            {
                /*Turn on timer 2 to interrupt on dwell time*/
                //No of cycles in 1 ms
                int noOfPeriodsInMS = (int)(0.001*pbBusFreq);

                //find prescaler which gives the most accurate time resolution
                int prescaler = 1;
                if (noOfPeriodsInMS*dwellTime < 65536)
                {
                    T2CONbits.TCKPS = 0;    //Set prescaler to 1:
                    prescaler = 1;
                }
                if (noOfPeriodsInMS*dwellTime < 131072)
                {
                    T2CONbits.TCKPS = 1;    //Set prescaler to 2:
                    prescaler = 2;
                }
                if (noOfPeriodsInMS*dwellTime < 262144)
                {
                    T2CONbits.TCKPS = 2;    //Set prescaler to 4:
                    prescaler = 4;
                }
                else if (noOfPeriodsInMS*dwellTime < 524288)
                {
                    T2CONbits.TCKPS = 3;    //Set prescaler to 1:8
                    prescaler = 8;
                }
                else if (noOfPeriodsInMS*dwellTime < 1048576)
                {
                    T2CONbits.TCKPS = 4;    //Set prescaler to 1:16
                    prescaler = 16;
                }
                else if (noOfPeriodsInMS*dwellTime < 2097152)
                {
                    T2CONbits.TCKPS = 5;    //Set prescaler to 1:32
                    prescaler = 32;
                }
                else if (noOfPeriodsInMS*dwellTime < 4194304)
                {
                    T2CONbits.TCKPS = 6;    //Set prescaler to 1:64
                    prescaler = 64;
                }
                else if (noOfPeriodsInMS*dwellTime < 16777216)
                {
                    T2CONbits.TCKPS = 7;    //Set prescaler to 1:256
                    prescaler = 256;
                }
                //Maximum delay at 50MHz is 335ms

                //Initialize timer 2
                IFS0bits.T2IF = 0;                                              //Clear interrupt flag
                TMR2 = 65536 - (int)((noOfPeriodsInMS/prescaler)*dwellTime);    //Set timer 1 register to appropriate delay
                T2CONbits.ON = 1;                                               //Turn on timer 2


            }
            while(IFS0bits.T2IF == 0)
            {
                /*Read RSSI*/
                rawRSSI = readRFM22BRegister(0x26);
                /*Changes rssi bit value to dBm*/
                unCalRssi = 0.5*rawRSSI - 122.5;
                /*Apply calibration*/
                calRssi = unCalRssi;

                /*Store values*/
                if(calRssi > maxSample)
                    maxSample = calRssi;
                else if (calRssi < minSample)
                    minSample = calRssi;

                averageSample += calRssi;

                /*Acumulate Samples for dwell time period*/
                currentSample = currentSample + 1;

            }
           
            averageSample = averageSample/currentSample;

            sprintf(&packet,"%.1f %.1f %.1f",minSample,averageSample,maxSample);
            transmitToHost(packet);
            ledState = STATE_IDLE;
            setRGBState(STATE_IDLE);
            receiveBuffer[0] = "\0";
            count = 0;
            currentSample = 0;
            minSample = 0;
            maxSample = -130;
            averageSample = 0;

            //Turn off timer 2
            IFS0bits.T2IF = 0;                                              //Clear interrupt flag
            T2CONbits.ON = 0;                                               //Turn off timer 2
            
            break;
    }
}

/*Sets up the radio the specified radio frequency
 240MHz -> 960MHz, frequency is specified in MHz*/
void setRadioFrequency(double freq)
{
    //First generate the neccesary setting for the specified frequency
    char bandSelect = 0b01000000;
    char fb = 0b00000000;
    short fc = 0;
    int xtalfreq = 30000;


    if(freq >= 480)
    {
        bandSelect |= 0b00100000;
        fb = (int)floor((freq)/(10*(xtalfreq/30000)*(1+1))) - 24;
        double temp = freq/(10*(xtalfreq/30000)*(1+1));
        fc = (short)floor((temp -floor(temp) )*64000 + 0.49999);
    }
    else 
    {
        fb = (int)floor((freq)/(10*(xtalfreq/30000)*(1+0))) - 24;
        double temp = freq/(10*(xtalfreq/30000)*(1+0));
        fc = (short)floor((temp -floor(temp) )*64000 + 0.49999);
    }
    bandSelect |= fb;
    char fc1 = (char)(fc>>8);
    char fc2 = (char)(fc&0b0000000011111111);

    /*Write values to RFM22B*/
    writeRFM22BRegister(bandSelect,0x75);
    writeRFM22BRegister(fc1,0x76);
    writeRFM22BRegister(fc2,0x77);

}

/*Configures the radio to default settings used for
 spectrum analyser mode*/
void initRadio()
{
     //Set Antenna to receive mode
     //RX  -- RB9
     //TX  -- RB5
     PORTClearBits(IOPORT_B,BIT_5);
     PORTSetBits(IOPORT_B,BIT_9);

     //Turn on PLL to enter tune mode
     //Turn RX on to enter receive mode
     writeRFM22BRegister(0b00000110,0x07);
     //Turn off AGC and set LNA gain to 5dB minimum and pga gain to 0
     writeRFM22BRegister(0b00000000,0x69);
}


/*Read a specified register from the RFM22B module via
 the SPI communication channel*/
char readRFM22BRegister(int address)
{
    char data = 'X';
    int dummyByte = 0xFF;
    /*Reset first byte refection counter*/
    spiCount = 0;
    /*Set chip select line for rfm22b*/
    PORTClearBits(IOPORT_B,BIT_7);
    /*Wait some time for rfm22b to get ready*/
    delay_us(10);
    /*Send address to read and wait for buffer to clear*/
    SpiChnPutS(SPI_CHANNEL2, &address,1);
    while(!SpiChnDataRdy(SPI_CHANNEL2)){}
    data = SpiChnGetC(SPI_CHANNEL2);

    /*Send dummy byte and wait for rfm22b reply interrupt which will store
     the data in the spiBuffer variable*/    
    SpiChnPutS(SPI_CHANNEL2, &dummyByte,1);
    while(!SpiChnDataRdy(SPI_CHANNEL2)){}
    data = SpiChnGetC(SPI_CHANNEL2);

    /*turn off communication to rfm22b and reset spi buffer state*/
    PORTSetBits(IOPORT_B,BIT_7);
    /*Return requested data*/
    return data;
}

/*Writes a specified register from the RFM22B module via
 the SPI communication channel*/
void writeRFM22BRegister(char data,char address)
{

    /*Reset first byte refection counter*/
    spiCount = 0;
    /*Set chip select line for rfm22b*/
    PORTClearBits(IOPORT_B,BIT_7);
    /*Append write bit to specify a write a action*/
    address |= 0b10000000;
    /*Wait some time for rfm22b to get ready*/
    delay_us(10);
    /*Send address to read and wait for buffer to clear*/
    SpiChnPutS(SPI_CHANNEL2, &address,1);
    while(!SpiChnDataRdy(SPI_CHANNEL2)){}
    SpiChnGetC(SPI_CHANNEL2);

    /*Send dummy byte and wait for rfm22b reply interrupt which will store
     the data in the spiBuffer variable*/
    SpiChnPutS(SPI_CHANNEL2, &data,1);
    while(!SpiChnDataRdy(SPI_CHANNEL2)){}
    data = SpiChnGetC(SPI_CHANNEL2);

    /*turn off communication to rfm22b and reset spi buffer state*/
    PORTSetBits(IOPORT_B,BIT_7);
}


/*Checks the buffer state and handles events accordingly*/
void checkBufferState()
{
    int badCommand = 0;
    double jumpSize = 0;
    char str[30];

    switch(bufferState)
    {
        case BUFFER_READY:
            /*Check the type of command*/
            if(receiveBuffer[0] == 'R')/*Sweep command <R.300000000.400000000.100000>*/
            {
                startFrequency = 0;
                stopFrequency = 0;
                rbw = 0;

                /*Set state and status indicator*/
                ledState = STATE_SWEEP;
                setRGBState(STATE_SWEEP);

                /*Extract frequency out of command*/
                /*Break up command and store components*/
                char * pch;
                pch = strtok (receiveBuffer,".");
                pch = strtok (NULL, ".");
                while (pch != NULL)
                {
                    if(startFrequency == 0)
                    {
                            startFrequency = (double)(atol(pch))/1000000;
                    }
                    else if(stopFrequency == 0)
                        stopFrequency = (double)(atol(pch))/1000000;
                    else
                    {
                        rbw = (double)(atol(pch))/1000;
                    }

                    pch = strtok (NULL, ".");
                    
                }
                freqStep = (stopFrequency - startFrequency)/noOfSteps;
                currentFrequency = startFrequency;

                /*Setup IF Filter with the use of the lookuptable*/
                if(setIFFilter(rbw))
                {
                    badCommand = 1;
                }

                bufferState = BUFFER_IDLE;

            }
            if(receiveBuffer[0] == 'D')/*Single frequency command <D.433000000.021000.000001000>*/
            {
                startFrequency = 0;
                stopFrequency = 0;
                rbw = 0;
                dwellTime = 0;
                /*Set state and status indicator*/
                ledState = STATE_DISCRETE;
                setRGBState(STATE_DISCRETE);

                /*Extract frequency out of command*/
                /*Extract frequency out of command*/
                /*Break up command and store components*/
                char * pch;
                pch = strtok (receiveBuffer,".");
                pch = strtok (NULL, ".");
                while (pch != NULL)
                {
                    if(startFrequency == 0)
                        startFrequency = (double)(atol(pch))/1000000;
                    else if(rbw == 0)
                        rbw = (double)(atol(pch))/1000;
                    else if(dwellTime == 0)
                        dwellTime = (int)(atol(pch));
                    pch = strtok (NULL, ".");
                }
                currentFrequency = startFrequency;
                if(dwellTime < 0 || dwellTime > 335)
                    badCommand = 1;

                /*Setup IF Filter with the use of the lookuptable*/
                if(setIFFilter(rbw))
                {
                    badCommand = 1;
                }

                bufferState = BUFFER_IDLE;
                frequencyHasBeenSet = 0;

            }
            if(receiveBuffer[0] == 'S')/*Settings command <S.1.5.11.###################>*/
            {
                lnaGain = -1;
                progGain = -1;
                noOfSteps = -1;
                /*Set gain settings*/
                /*LNA Gain
                 *0 - 5dB
                 *1 - 25dB
                 *0 - 000 - 0dB
                 *1 - 001 - 3dB
                 *2 - 010 - 6dB
                 *8 - 101 - 24dB Max*/
                /*Extract frequency out of command*/
                /*Extract frequency out of command*/
                /*Break up command and store components*/
                char * pch;
                pch = strtok (receiveBuffer,".");
                pch = strtok (NULL, ".");
                while (pch != NULL)
                {
                    if(lnaGain == -1)
                        lnaGain = atoi(pch);
                    else if(progGain == -1)
                        progGain = atoi(pch);
                    else if(noOfSteps == -1)
                        noOfSteps = atoi(pch);

                    pch = strtok (NULL, ".");
                }
                /*Check for errors*/
                if(lnaGain == -1 || progGain == -1 || lnaGain > 1 || progGain > 5)
                {
                    transmitToHost("[Settings FAIL]");
                    badCommand = 1;
                }
                else
                {
                    /*Set gain settings*/
                    char gainReg = 0b00000000;
                    gainReg |= (lnaGain<<4);
                    gainReg |= progGain;

                    /*Write radio gain settings*/
                    writeRFM22BRegister(gainReg,0x69);

                    /*Check if step size changed, if so reset trace with new step size*/
                    if(!(((stopFrequency - startFrequency)/noOfSteps) == freqStep))
                    {
                        freqStep = (stopFrequency - startFrequency)/noOfSteps;
                        currentFrequency = startFrequency;
                    }
                    transmitToHost("[Settings OK]");
                }

            }
            else if(receiveBuffer[0] == 'H')
            {
                /*Handle Polling Event, reset state to idle and say hello back*/
                ledState = STATE_IDLE;
                setRGBState(STATE_IDLE);
                transmitToHost("[Hello]");
                bufferState = BUFFER_IDLE;
            }
            else
                badCommand = 1;

            if(badCommand) /*bad command received*/
            {
                receiveBuffer[0] = "\0";
                count = 0;
                bufferState = BUFFER_ERROR;
            }

            bufferState = BUFFER_IDLE;
            break;

        case BUFFER_ERROR:
            ledState = STATE_ERROR;
            setRGBState(STATE_ERROR);
            transmitToHost("[Retry]");
            bufferState = BUFFER_IDLE;
            break;
    }
}

/*Setup RFM22B Registers to values closest to those satisfying the specified 
 resolution bandwidth by use of a lookup filter*/
int setIFFilter(double rbw)
{
    /*Run through filter list and see which filter index was chosen*/
    int index = -1;
    int i = 0;
    for(i = 0;i < sizeof(IFfilterLookup)/sizeof(int) && index == -1;i++)
    {
        if(rbw == IFfilterLookup[i])
            index = i;
    }
    if(index == -1)
        return 1;

    /*Construct register out of lookuptable*/
     char ifReg = 0b00000000;
     ifReg |= (IFfilterDwn3Bypass[index]<<7);
     ifReg |= (IFfilterNDecExp[index]<<4);
     ifReg |= (IFfilterFilset[index]);

     /*Write register to radio*/
     writeRFM22BRegister(ifReg,0x1C);

//     char str[10];
//     sprintf(&str,"%d",ifReg);
//     transmitToHost(str);
//     while(1){}
     return 0;

}

/*Initializes the state of the RGB LED pins and sets the startup state*/
void initStatusLED()
{
    //Initialize status RGB LED
    //Red   - RB3
    //Blue  - RA1
    //Green - RB13
    //Turn off LED before port is initialized
    PORTSetBits(IOPORT_B,BIT_3|BIT_13);
    PORTSetBits(IOPORT_A,BIT_1);
    //Set tristate buffer to outputs
    PORTSetPinsDigitalOut(IOPORT_B,BIT_3|BIT_13);
    PORTSetPinsDigitalOut(IOPORT_A,BIT_1);

    //Set state to Idle
    ledState = STATE_IDLE;
    setRGBState(STATE_IDLE);
    
    //Set RGB update timer
    // configure the rgb led to update its status at 23Hz
    //OpenTimer2(T2_ON | T2_PS_1_256, 0xFFFF);
    //OpenOC4( OC_ON | OC_TIMER_MODE16 | OC_TIMER2_SRC | OC_PWM_FAULT_PIN_DISABLE , 0, 0 );
   
    // set up the core timer interrupt with a prioirty of 2 and zero sub-priority
    //ConfigIntTimer2(T2_INT_ON | T2_INT_PRIOR_1);
    transmitToHost("[Status indicator OK]");
}

/*Sets the current RGB LED state indication*/
void setRGBState(int state)
{
    switch(state)
    {
        case STATE_IDLE:
            //Set RGB to blue
            //RPB13R = 0b0000;
            PORTSetBits(IOPORT_B,BIT_13|BIT_3);
            PORTClearBits(IOPORT_A,BIT_1);
            break;

        case STATE_SWEEP:
            //Set RGB to green
            PORTSetBits(IOPORT_B,BIT_3);
            PORTSetBits(IOPORT_A,BIT_1);
            PORTClearBits(IOPORT_B,BIT_13);
            //Animate LED
//            RPB13R = 0b0101;
//            SetDCOC4PWM(ledIntensity); // unsigned int duty from 0-1000
//            delay_ms(40);
//            if(up)
//                ledIntensity = ledIntensity + ledStep;
//            else
//                ledIntensity = ledIntensity - ledStep;
//            if(up && ledIntensity > 0xFFFF-ledStep + 1)
//                up = 0;
//            else if(!up && ledIntensity < ledStep)
//                up = 1;
            break;

        case STATE_DISCRETE:
            //Set RGB to flashing green
            //RPB13R = 0b0000;
            PORTSetBits(IOPORT_B,BIT_3);
            PORTClearBits(IOPORT_A,BIT_1);
            PORTClearBits(IOPORT_B,BIT_13);
         
//            discreteFlashCounter += 1;
//            if(discreteFlashCounter == DISCRETE_FLASH_PERIOD)
//            {
//                discreteFlashCounter = 0;
//                if(distreteFlashState == 0)
//                {
//                    PORTSetBits(IOPORT_B,BIT_13);
//                    distreteFlashState = 1;
//                }
//                else
//                {
//                    PORTClearBits(IOPORT_B,BIT_13);
//                    distreteFlashState = 0;
//                }
//            }
            break;

        case STATE_ERROR:
            //Set RGB to red
            //RPB13R = 0b0000;
            PORTSetBits(IOPORT_B,BIT_13);
            PORTSetBits(IOPORT_A,BIT_1);
            PORTClearBits(IOPORT_B,BIT_3);
            break;
    }
}


/*Initialize all communication channels between host computer as well as rf
 radios
 Communication to host is at 56kbps through UART Port 1
 Communication to RFM Radios are through SPI 1
 Communication to radios also get tested by reading their version numbers,
 if a radio cannot be detected the system will go into a error mode*/
void initCommunications()
{
    int data =5;
    char aChar = " ";

    //Initialize UART Port with interrupts to detect incomming commands
    //Map Uart peripherals to pins
    //RA0 = UART TX
    //RA4 = UART RX
    PORTSetPinsDigitalOut(IOPORT_A,BIT_0);
    PORTSetPinsDigitalIn(IOPORT_A,BIT_4);
    RPA0R = 0b0001;
    U1RXR = 0b0010;

    //Open UART
    // at 9600 baud - 162
    // at 56000 baud - 27
    UARTConfigure(UART1, UART_ENABLE_PINS_TX_RX_ONLY);
    UARTSetFifoMode(UART1, UART_INTERRUPT_ON_RX_NOT_EMPTY);
    INTSetVectorPriority( INT_UART_1_VECTOR,INT_PRIORITY_LEVEL_2);
    INTSetVectorSubPriority( INT_UART_1_VECTOR,INT_SUB_PRIORITY_LEVEL_2);
    INTClearFlag(INT_U1RX);
    // Enable the interrupt source
    INTEnable(INT_U1RX, INT_ENABLED);
    UARTSetLineControl(UART1, UART_DATA_SIZE_8_BITS | UART_PARITY_NONE| UART_STOP_BITS_1);
    long uartBaud = UARTSetDataRate(UART1, pbBusFreq, 19200);
    UARTEnable(UART1, UART_ENABLE_FLAGS(UART_PERIPHERAL | UART_RX | UART_TX));
    transmitToHost("[UART communication OK]");

    /*Initialize visual indications*/    
    initStatusLED();

    //Initialize SPI port
    //Setup Pins
    //Slave select -- B7
    RPB8R = 0b0100; //SDO -- B8
    SDI2R = 0b0100;  //SDI -- B2
    //SCK -- RB15
    //RX  -- RB9
    //TX  -- RB5
    PORTSetBits(IOPORT_B,BIT_7);
    PORTSetPinsDigitalOut(IOPORT_B, BIT_8|BIT_15|BIT_7|BIT_9|BIT_5);
    PORTSetPinsDigitalIn(IOPORT_B, BIT_2);

    /*Setup SPI peripheril at 25MHz*/
    SpiChnOpen(SPI_CHANNEL2, SPI_OPEN_MSTEN|SPI_OPEN_SMP_END|SPI_OPEN_CKE_REV  |SPI_OPEN_MODE8, 2);
    SpiChnEnable(SPI_CHANNEL2, 1);// will configure and enable the SPI2 channel.

   


    /*Test radio and send init sequence to host*/
    if(readRFM22BRegister(0x01) == 6) //Check version number
        transmitToHost("[Radio communication OK]");
    else
    {
        transmitToHost("[Radio communication FAIL]");
        ledState = STATE_ERROR;
        setRGBState(STATE_ERROR);
        while(1){} //Stop system
    }
    /*Set radio in receive mode*/
    initRadio();

}

/*Transmit to host function, checks if transmission is possible and sends
 the given data to the host*/
void transmitToHost(char* string)
{
    /*Check if UART is not still busy transmitting previous payload,
     if busy wait until the transmit is finished*/
    while(!UARTTransmitterIsReady(UART1)){}
        UARTSendDataString(UART1, string);
    putcUART1('\r');
    putcUART1('\n');
}

/*Make the processor wait for a certain time specified in multiples of ms*/
void delay_ms(int i)
{
    //No of cycles in 1 ms
    int noOfPeriodsInMS = (int)(0.001*pbBusFreq);

    //find prescaler which gives the most accurate time resolution
    int prescaler = 1;
    if (noOfPeriodsInMS*i < 65536)
    {
        T1CONbits.TCKPS = 0;    //Set prescaler to 1:
        prescaler = 1;
    }
    else if (noOfPeriodsInMS*i < 524288)
    {
        T1CONbits.TCKPS = 1;    //Set prescaler to 1:8
        prescaler = 8;
    }
    else if (noOfPeriodsInMS*i < 4194304)
    {
        T1CONbits.TCKPS = 2;    //Set prescaler to 1:64
        prescaler = 64;
    }
    else if (noOfPeriodsInMS*i < 16777216)
    {
        T1CONbits.TCKPS = 3;    //Set prescaler to 1:256
        prescaler = 256;
    }
    //Maximum delay at 50MHz is 385ms


    //Initialize timer 1
    IFS0bits.T1IF = 0;                                      //Clear interrupt flag
    TMR1 = 65536 - (int)((noOfPeriodsInMS/prescaler)*i);    //Set timer 1 register to appropriate delay
    T1CONbits.ON = 1;                                       //Turn on timer 1

    //Wait for timer to finish
    while(!IFS0bits.T1IF){}

    //Turn off timer 1
    T1CONbits.ON = 0;
}

/*Make the processor wait for a certain time specified in multiples of us*/
void delay_us(int i)
{
    //No of cycles in 1 ms
    int noOfPeriodsInUS = (int)(0.000001*pbBusFreq);

    //find prescaler which gives the most accurate time resolution
    int prescaler = 1;
    if (noOfPeriodsInUS*i < 65536)
    {
        T1CONbits.TCKPS = 0;    //Set prescaler to 1:
        prescaler = 1;
    }
    else if (noOfPeriodsInUS*i < 524288)
    {
        T1CONbits.TCKPS = 1;    //Set prescaler to 1:8
        prescaler = 8;
    }
    else if (noOfPeriodsInUS*i < 4194304)
    {
        T1CONbits.TCKPS = 2;    //Set prescaler to 1:64
        prescaler = 64;
    }
    else if (noOfPeriodsInUS*i < 16777216)
    {
        T1CONbits.TCKPS = 3;    //Set prescaler to 1:256
        prescaler = 256;
    }
    //Maximum delay at 25MHz is 670000us

    //Initialize timer 1
    IFS0bits.T1IF = 0;                                      //Clear interrupt flag
    TMR1 = 65536 - (int)((noOfPeriodsInUS/prescaler)*i);    //Set timer 1 register to appropriate delay
    T1CONbits.ON = 1;                                       //Turn on timer 1

    //Wait for timer to finish
    while(!IFS0bits.T1IF){}

    //Turn off timer 1
    T1CONbits.ON = 0;
}
// Timer 2 interrupt handler
// it is set at priority level 1
void __ISR(_TIMER_2_VECTOR, ipl1) updateStatusLED(void)
{
    //setRGBState(ledState);
    mT2ClearIntFlag(); // clear the interrupt flag
}

// UART 1 interrupt handler
// it is set at priority level 2
void __ISR(_UART_1_VECTOR, ipl2) IntUart1Handler(void)
{
	// Is this an RX interrupt?
	if(INTGetFlag(INT_U1RX))
	{
	    //Clear the RX interrupt Flag
	    INTClearFlag(INT_U1RX);
            /*Handle received data by adding it to the receive buffer.
             Check for the start and end of command characters.
             If a start character is received, flush the buffer,
             if a end character is received, acknowledge the data and
             raise a received command flag*/
            while(UARTReceivedDataIsAvailable(UART1))
            {
                char receivedByte = UARTGetDataByte(UART1);
                //UARTSendDataByte(UART1,receivedByte);
                if(receivedByte == '<') /*Start of command indicator*/
                {
                    /*Flush buffer*/
                    receiveBuffer[0] = "\0";
                    count = 0;
                    bufferState = BUFFER_IDLE;
                }
                else if(receivedByte == '>') /*End of command indicator*/
                {
                    /*Check if buffer has correct length*/
                    if(count != COMMAND_LENGTH)
                    {
                         /*Flush buffer and raise error flag*/
                         receiveBuffer[0] = "\0";
                         count = 0;
                         bufferState = BUFFER_ERROR;
                    }
                    else
                    {
                        /*Notify system a valid command is ready*/
                        bufferState = BUFFER_READY;

                    }
                }
                else
                {
                    if(count <= COMMAND_LENGTH)/*Check if command is still within limits*/
                    {
                        /*Add byte to buffer*/
                        receiveBuffer[count++]= receivedByte;
                        
                    }
                    else
                    {
                        bufferState = BUFFER_ERROR;

                    }
                }

            }


	}

}