#TODO
#Add command line inputs
#Add help display
#Read in byte format
#Make plot interactive
#Add timestamp to discrete output
#Add max and min hold to sweep output
#Read buffer before processing strings
#Auto calculate no of points

import sys
import matplotlib.pyplot as plt
import numpy as np 
import serial as sr
import re
import time as t

#EMU (Electromagnetic measurement device) Plotter
#This program extracts data from the device through the ftdi port
#and displays the data on respective plots

#Returns current time as string
def getTimeStamp():
	return str(t.localtime().tm_hour) +":"+ str(t.localtime().tm_min) +":"+ str(t.localtime().tm_sec)

#Adds neccesary padding and delimeters to command
def encapsulateCommand(command):
	commandLength = 28
	
	#Add padding to command
	commaAdded = False
	while len(command) < 28:
		if not commaAdded:
			command = command + "."
			commaAdded = True
		else:
			command = command + "0"
	
	#Add start and stop delimeters
	command = "<"+command+">"
			
	return command		


#Check if help was called or zero arguments were given
if len(sys.argv) == 1 or sys.argv[1] == "-help" or sys.argv[1] == "-h":
	#Print out usage instructions
	print("\nEMU Plot instructions:\n")
	
	print("EMUPlot.py [Type] [Frequency] [RBW] [Gain] [Dwell]\n")
	
	print("[Type]: Sweep - 	Sweeps a given set of frequencies and") 
	print("			outputs to a cartesian graph")
	print("        Disc  - 	Measures a given set of discrete frequencies") 
	print("			and outputs the an array as a max, average or") 
	print("			min values\n")

	print("[Frequencies]:   Sweep: -ss startFrequency stopFrequency (MHz)")
	print("                        -cs centerFrequency span         (MHz)")
	print("                 Disc:  -noOfFrequencies freq1 ... freqN (MHz)\n")
	
	print("[RBW]: Resolution bandwith [Hz]")
	print("       use -rbw specifier to list available resolution bandwidths")
	sys.exit()
	
#Check that input is valid and assign parameters
if (not "Sweep" == str(sys.argv[1])) and (not "Disc" == str(sys.argv[1])):
	print("Measure type argument invalid [Sweep, Disc]")
	sys.exit() 
type = str(sys.argv[1])		#Type of measurement (Sweep or Disc)	

if(type == "Sweep"):
	#Read in frequency set
	#Check -ss or -cs identifier
	if(len(sys.argv) < 5):
		print("Frequency specifier incomplete, see help (-h or -help)")
		sys.exit()
	if(sys.argv[2] == "-ss"):
		startFreq = float(sys.argv[3])*1000000
		stopFreq = float(sys.argv[4])*1000000
	elif(sys.argv[2] == "-cs"):
		startFreq = float(sys.argv[3])*1000000 - float(sys.argv[4])*1000000/2	
		stopFreq = float(sys.argv[3])*1000000 + float(sys.argv[4])*1000000/2
	else:
		print ("Frequency set specifier invalid, see help (-h or -help)")
	if(startFreq < 240000000 or stopFreq > 930000000):
		print("Frequency set out of range (240MHz - 930MHz)")
		sys.exit()
elif(type == "Disc"):
	#Read in frequency array set
	if(len(sys.argv) < 3):
		print("Frequency specifier incomplete, see help (-h or -help)")
		sys.exit()
	noOfFreq = int(sys.argv[2])
	if(noOfFreq < 1):
		print("Cannot specify 0 or a negative number of frequencies")
		sys.exit()
	if(len(sys.argv) < 3 + noOfFreq):
		print("Frequency specifier incomplete, see help (-h or -help)")
		sys.exit()
	freqList = []
	for i in np.arange(0,noOfFreq):
		freqList.append(float(sys.argv[3 + i])*1000000)
	if(max(freqList) > 930000000 or min(freqList) < 240000000):
		print("Frequency in set out of range (240MHz - 930MHz)")
		sys.exit()

#Read in RBW
if(type == "Sweep"):
	if(len(sys.argv) < 6):
		print("Resolution bandwidth filter missing, see help (-h or -help)")
		sys.exit()
	else:
		rbw = int(sys.argv[6])
elif(type == "Disc"):
	if(len(sys.argv) < 3 + noOfFreq):
		print("Resolution bandwidth filter missing, see help (-h or -help)")
		sys.exit()
	else:
		rbw = int(sys.argv[3 + noOfFreq])
#Check if rbw is in compatibility list


#Read in Gains

#Read in Dwell time

#Calculate no of points

if(type == "Sweep"):
	startFreq = int(startFreq)
	stopFreq = int(stopFreq)

#Print out selected parameters
print("************EMU Reader************")
print("Measure Type: "+type)
if(type == "Sweep"):
	print("Start Frequency [Hz]: "+str(startFreq))
	print("Stop Frequency  [Hz]: "+str(stopFreq))
elif(type == "Disc"):
	print("Frequency List  [Hz]: "+str(freqList))
print("Resolution bandwidth: "+str(rbw))
#Parameters
noOfPoints = 400
#rbw = 2600 			#Hz (2.6kHz, 21kHz, 620.7kHz)
lna = 1				#5dB
pga = 5	 			#0dB

#Discrete settings
#List of frequencies to scan (240MHz - 930MHz)
#freqList = [0.450e9, 300e6, 700e6]	
dwellTime = 10 	#Time in ms to listen to each signal

#Sweep settings
#span = 400000
#center = 300000000
#startFreq = center - span/2
#stopFreq = center + span/2
#startFreq = 240000000	#Sweep starting frequency
#stopFreq = 612420000	#Sweep stop frequency

#Connect serial port
port = sr.Serial("/dev/ttyUSB0",baudrate=20000)
port.flushInput()

#Check if EMU is connected and working
port.write(encapsulateCommand("Hello"))
if "[Hello]" in port.readline():
	print("EMU Detected")
else:
	print("Could not detect EMU")

#Set EMU settings for data request
print("Setting EMU settings")
port.write(encapsulateCommand("S."+str(lna)+"."+str(pga)+"."+str(noOfPoints)))
if not "OK" in port.readline():
	print("Configuration failed")

#Collect data from EMU
#Sweep script
if(type == "Sweep"):
	plt.ion()
	fig = plt.figure(1)
 	ax = fig.add_subplot(111)
	firstSweep = True
	while True:
		#Send sweep measure command
#		print(encapsulateCommand("R."+str(startFreq)+"."+str(stopFreq)+"."+str(rbw)))
		port.write(encapsulateCommand("R."+str(startFreq)+"."+str(stopFreq)+"."+str(rbw)))
		frequency = []
		signal = []
		for i in np.arange(0,noOfPoints):
			msg = port.readline()
			splitmsg = msg.split(' ')
			frequency.append(float(splitmsg[0])*1000000)
			signal.append(float(splitmsg[1]))
			print("Reading frequency: "+str(frequency[-1])+" "+str(signal[-1]))
		#Plot measured data
		print("Plotting data")
		if(firstSweep): 
			plt.title("EMU Spectrum")
			plt.xlabel('Frequency [Hz]')
			plt.ylabel('Signal [dBm]')
			plt.xlim(startFreq,stopFreq)
			plt.ylim(-120,0)	
			plt.grid(True)
			line, = plt.plot(frequency,signal,color='red')
			firstSweep = False		
		else:
			line.set_ydata(signal)	
		plt.draw()
#Discrete script
if(type == "Disc"):
	while True:
		frequency = []
		signalMin = []
		signalAve = []
		signalMax = []
		#Loop through frequencies
		for freqPoint in freqList:
			#Send discrete measure command
#			print("Measuring: "+str(freqPoint))
			port.write(encapsulateCommand("D."+str(freqPoint)+"."+str(rbw)+"."+str(dwellTime)))
			#Store received data
			frequency.append(freqPoint)
			msg = port.readline()
#			print(msg)
			msgSplit = msg.split(' ')
			signalMin.append(float(msgSplit[0]))
			signalAve.append(float(msgSplit[1]))
			signalMax.append(float(msgSplit[2]))
#			print(msg)
	#Print out data
		print(getTimeStamp() +" "+ str(signalAve))

print("EMU ended succesfully")

